<script setup lang="ts">
import type { Recordable } from '@vben/types';

import { computed, nextTick, ref } from 'vue';
import { useRoute } from 'vue-router';

import { useAccess } from '@vben/access';
import { Page, useVbenDrawer, type VbenFormProps } from '@vben/common-ui';
import { $t } from '@vben/locales';
import { getVxePopupContainer } from '@vben/utils';

import { Modal, Popconfirm, Space } from 'ant-design-vue';

import { useVbenVxeGrid, vxeCheckboxChecked, type VxeGridProps, vxeSortEvent } from '#/adapter/vxe-table';
import { delInventoryDetail, inventoryDetailExport, listInventoryDetail } from '#/api/eims/inventory-detail';
import { commonDownloadExcel } from '#/utils/file/download';

import { columns, querySchema } from './data';
import detailDrawer from './detail-drawer.vue';
import EquTypeTree from './equ-type-tree.vue';

const route = useRoute();
const inventoryId = route.params.inventoryId as string;

// 左边部门用
const selectTypeId = ref<string[]>([]);
// 已盘点
const check = ref<string>('已盘');
// 未盘点
const noCheck = ref<string>('未盘');
// 查询添加盘点标记过滤
const checkFilter = ref<string>('');

const formOptions: VbenFormProps = {
  commonConfig: {
    labelWidth: 80,
    componentProps: {
      allowClear: true
    }
  },
  schema: querySchema(),
  wrapperClass: 'grid-cols-1 md:grid-cols-2 lg:grid-cols-3 xl:grid-cols-4',
  handleReset: async () => {
    selectTypeId.value = [];
    checkFilter.value = '';
    // eslint-disable-next-line no-use-before-define
    const { formApi, reload } = tableApi;
    await formApi.resetForm();
    const formValues = formApi.form.values;
    formApi.setLatestSubmissionValues(formValues);
    await reload(formValues);
  },
  // 日期选择格式化
  fieldMappingTime: [['createTime', ['params[beginTime]', 'params[endTime]'], ['YYYY-MM-DD 00:00:00', 'YYYY-MM-DD 23:59:59']]]
};

const gridOptions: VxeGridProps = {
  checkboxConfig: {
    // 高亮
    highlight: true,
    // 翻页时保留选中状态
    reserve: true
    // 点击行选中
    // trigger: 'row'
  },
  columns,
  height: 'auto',
  keepSource: true,
  pagerConfig: {
    // 默认条数
    pageSize: 1000,
    // 分页可选条数
    pageSizes: [1000, 2000, 3000, 4000, 5000]
  },
  proxyConfig: {
    ajax: {
      query: async ({ page }, formValues = {}) => {
        // 部门树选择处理
        if (selectTypeId.value.length === 1) {
          formValues.equTypeId = selectTypeId.value[0];
        } else {
          Reflect.deleteProperty(formValues, 'equTypeId');
        }
        // 是否盘点
        if (checkFilter.value === '0' || checkFilter.value === '1') {
          formValues.status = checkFilter.value;
        } else {
          Reflect.deleteProperty(formValues, 'status');
        }
        return await listInventoryDetail({
          pageNum: page.currentPage,
          pageSize: page.pageSize,
          inventoryId,
          ...formValues
        });
      },
      querySuccess: () => {
        nextTick(() => {
          // 过滤已盘和未盘的数据
          // eslint-disable-next-line no-use-before-define
          const data = tableApi.grid.getData();
          const data0 = data.filter((item) => {
            return item.status === '' || item.status === null || item.status === '0';
          });
          // 不点击已盘和未盘时才更新已盘和未盘的统计数据
          if (checkFilter.value === '') {
            noCheck.value = data0 !== null && data0.length > 0 ? `未盘(${data0.length})` : `未盘(0)`;
            const data1 = data.filter((item) => {
              return item.status !== '' && item.status !== null && item.status === '1';
            });
            check.value = data1 !== null && data1.length > 0 ? `已盘(${data1.length})` : `已盘(0)`;
          }
        });
      }
    }
  },
  rowConfig: {
    isHover: true,
    keyField: 'id'
  },
  sortConfig: {
    // 远程排序
    remote: true,
    // 支持多字段排序 默认关闭
    multiple: true
  },
  id: 'eims-inventory-detail-index'
};

const [BasicTable, tableApi] = useVbenVxeGrid({
  formOptions,
  gridOptions,
  gridEvents: {
    sortChange: (sortParams) => vxeSortEvent(tableApi, sortParams)
  }
});

const [DetailDrawer, detailDrawerApi] = useVbenDrawer({
  connectedComponent: detailDrawer
});

async function handleEdit(record: Recordable<any>) {
  detailDrawerApi.setData({ id: record.id });
  detailDrawerApi.open();
}

async function handleDelete(row: Recordable<any>) {
  await delInventoryDetail(row.id);
  await tableApi.query();
}

function handleMultiDelete() {
  const rows = tableApi.grid.getCheckboxRecords();
  const ids = rows.map((row: any) => row.id);
  Modal.confirm({
    title: '提示',
    okType: 'danger',
    content: `确认删除选中的${ids.length}条记录吗？`,
    onOk: async () => {
      await delInventoryDetail(ids);
      await tableApi.query();
    }
  });
}

function handleDownloadExcel() {
  commonDownloadExcel(inventoryDetailExport, '盘点明细', tableApi.formApi.form.values, {
    fieldMappingTime: formOptions.fieldMappingTime
  });
}

function handleQuery(isChecked: string) {
  checkFilter.value = isChecked;
  tableApi.query();
}

const { hasAccessByRoles } = useAccess();
const isSuperAdmin = computed(() => hasAccessByRoles(['superadmin']));
</script>

<template>
  <Page :auto-content-height="true">
    <div class="flex h-full gap-[8px]">
      <EquTypeTree v-model:select-dept-id="selectTypeId" class="w-[260px]" @reload="() => tableApi.reload()" @select="() => tableApi.reload()" />
      <BasicTable class="flex-1 overflow-hidden" table-title="盘点明细列表">
        <template #toolbar-tools>
          <Space>
            <a-button @click="handleQuery('0')">{{ noCheck }}</a-button>
            <a-button type="primary" @click="handleQuery('1')"> {{ check }}</a-button>
            <a-button v-access:code="['eims:inventory:export']" @click="handleDownloadExcel">
              {{ $t('pages.common.export') }}
            </a-button>
            <a-button
              :disabled="!vxeCheckboxChecked(tableApi) || !isSuperAdmin"
              danger
              type="primary"
              v-access:code="['eims:inventory:remove']"
              @click="handleMultiDelete"
            >
              {{ $t('pages.common.delete') }}
            </a-button>
          </Space>
        </template>

        <template #action="{ row }">
          <Space>
            <ghost-button :disabled="!isSuperAdmin" v-access:code="['eims:inventory:edit']" @click.stop="handleEdit(row)">
              {{ $t('pages.common.edit') }}
            </ghost-button>
            <Popconfirm :get-popup-container="getVxePopupContainer" placement="left" title="确认删除？" @confirm="handleDelete(row)">
              <ghost-button :disabled="!isSuperAdmin" danger v-access:code="['eims:inventory:remove']" @click.stop="">
                {{ $t('pages.common.delete') }}
              </ghost-button>
            </Popconfirm>
          </Space>
        </template>
      </BasicTable>
    </div>
    <DetailDrawer @reload="tableApi.query()" />
  </Page>
</template>
